// Global header file file for
// simple DMX512 controller for STM32F042F6 (20 pin TSSOP package)
// based on USB MIDISTREAM and CDC-ABSTRACT CONTROL MODEL
//
// Author and (c)2017-20 Wolfgang Schemmert.
// The copyright claim and GPL-3 licence is restricted to this file
// and the depending files "main.c","main.h","usb-fs-042.c", "dmx.c"
// and "userinterface.c".
// Other depending material is published and copyrighted
// by STMicroelectronics and Segger GmbH/Rowley Associates Ltd
//
// Contact: <www.midi-and-more.de>
// A detailled hardware construction and operation manual
// is provided at this website
//
// ****************************************************************************
//      This program is free software: you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//      Licence: GPL-3
//
// ****************************************************************************

#ifndef GLOBALDEFINES
#define GLOBALDEFINES

#define HARDWARE_VERSION 1  //describes project specific hardware version
#define REVNUM 5            //describes firmware revision
//REVNUM2, 11 OCT 2017:
// some features added,
// command codes aliged with project UsbEtherDMX-F042
//REVNUM3, 13 OCT 2017:
//bug fix user defined Vid/Pid: Hi and Low bytes exchanged
//REVNUM4, 23 July 2020
//bugfixes set DMX Level by MIDI CC and "all DMX to zero"
//improvements in user interface and Flash handling
//increased number of lighting scenes stored in Flash
//Fadetime not stored in Flash anymore, always start = 0
//released 31July2020
//REVNUM 5, 15 OCT 2021
//only HSI oscillator USB synchronzed is supported (doesn't work w.STM32F070)
//some additional commands, adjusted for compatibility with STM32G431 version
//USB MIDI transmission changed to "enDodeMidi()", supports SysEx too
//automatic USB enumeration on CDC/MIDI change disabled (lack of Flash)
//s
//
// ###########################################################################

#define CDC_CLASS   0       //initatalizes device as virtual COM port
#define MIDI_CLASS  1       //initatalizes device as native MIDI interface

#define DEFAULT_VIDH	0x04
#define DEFAULT_VIDL	0x83
#define DEFAULT_VID		((DEFAULT_VIDH << 8) | DEFAULT_VIDL)
#define DEFAULT_PIDH	0x57
#define DEFAULT_PIDL	0x40
#define DEFAULT_PID		((DEFAULT_PIDH << 8) | DEFAULT_PIDL)

#define DMX_BUFLEN 512 // index 0 is DMX addr. 1, TX from index 0 to 511

//#define DMX_DEBUG //config. DMX OUT provisionally as RS232 transmitter 115.2k
                  	//only possibility to debug USB MIDI_CLASS
					//and is helpful when USB crashes to get last debug messages
					
#define CTRL_B	2
#define BKSP	8
#define TAB		9
#define LFF		10		//exchange num.values of CRR and LFF for Macintosh OS
#define CRR		13
#define CTRL_R	18
#define ESC		27
#define SPACE	32
#define BACKSLASH 92

//not relevant for this project, but necessary for compilation:
#define EOX     0xF7
//#define RUNNINGSTATE  //uncomment if not wanted
//works only on MIDI msgs which are transmitted from MIDI OUT
//Running State is always supported with MIDI msgs received at MIDI IN

//bitmask shortcuts:
#define m0      1
#define m1      (1<<1)
#define m2      (1<<2)
#define m3      (1<<3)
#define m4      (1<<4)
#define m5      (1<<5)
#define m6      (1<<6)
#define m7      (1<<7)
#define m8      (1<<8)
#define m9      (1<<9)
#define m10     (1<<10)
#define m11     (1<<11)
#define m12     (1<<12)
#define m13     (1<<13)
#define m14     (1<<14)
#define m15     (1<<15)
#define m16     (1<<16)
#define m17     (1<<17)
#define m18     (1<<18)
#define m19     (1<<19)
#define m20     (1<<20)
#define m21     (1<<21)
#define m22     (1<<22)
#define m23     (1<<23)
#define m24     (1<<24)
#define m25     (1<<25)
#define m26     (1<<26)
#define m27     (1<<27)
#define m28     (1<<28)
#define m29     (1<<29)
#define m30     (1<<30)
#define m31     (1<<31)

#endif
