// USB-DMX-F042  ***************************************************************
//
// Simple DMX512 controller for STM32F042F6 (20 pin TSSOP package)
// based on USB MIDISTREAM and CDC-ABSTRACT CONTROL MODEL
//
// supports the MiniDMX protocol
// plus simple MIDI and ASCII string based command sets
//
// Author and (c)2017-21 Wolfgang Schemmert.
// The copyright claim and GPL-3 licence is restricted to this file
// and the depending files "main.h","usb-fs-042.c","dmx.c","userinterface.c"
// and "globaldefines.h".
// Other depending material is published and copyrighted
// by STMicroelectronics and Segger GmbH/Rowley Associates Ltd
//
// Contact: <www.midi-and-more.de>
// A detailled hardware construction
// and operation manual is provided at this website
//
// ****************************************************************************
//      This program is free software: you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//      Licence: GPL-3
//
// ****************************************************************************

#include "stm32f0xx.h"
#include "globaldefines.h"
#include "main.h"

int main(void)    // ********************************************************
// **************************************************************************
{
uint32_t recvword;
uint8_t miniDmx;
uint8_t recvbyte, index, status, data1, data2;

    RCC->AHBENR |= RCC_AHBENR_GPIOAEN;  // Enable Port A clock
    RCC->AHBENR |= RCC_AHBENR_GPIOBEN;  // Enable Port B clock

//   	for (recvword=0; recvword<1000000; recvword++); //clock start delay

    GPIOA->MODER = 0x68000000;      //all input except PA15,14,13
    GPIOB->MODER = 0x00000000;      //primarily all input
    GPIOA->PUPDR = 0x9682AAAA;      //primarily all inputs pulldown
									//exceptPA14,PA13(PullUp) and PA10,PA9
    GPIOB->PUPDR = 0xAAAAAAAA;      //primarily all inputs pulldownx

//pin config repeated specifically:
// PA3 enables/disables the DMX driver
    GPIOA->MODER &= ~(GPIO_MODER_MODER3);
    GPIOA->MODER |= GPIO_MODER_MODER3_0 ;	//PortA3 general purpose output mode
    GPIOA->BSRR = GPIO_BSRR_BR_3;		//clear bit PA3 (DMX driver disabled)
//Configure PA4 as LED driver
    GPIOA->MODER &= ~(GPIO_MODER_MODER4);
    GPIOA->MODER |= GPIO_MODER_MODER4_0;//set PortA4 (LED) as output
    GPIOA->BSRR =  GPIO_BSRR_BS_4;		//PA4 (LED) ON
// for USB, remap PA11,12 to PA9,10
    RCC->APB2ENR |= RCC_APB2ENR_SYSCFGCOMPEN;
    SYSCFG->CFGR1 |= SYSCFG_CFGR1_PA11_PA12_RMP;

    gSYSCOUNT = 20;
    gLEDCOUNT = 0;
    gDELAYCOUNT = 0;
    gTIMEOUT = 0;
    gVid = DEFAULT_VID;
    gPid = DEFAULT_PID;
    guDeviceClass = CDC_CLASS;
	gChannel = 0;
	gdFadeTime = 0;
	gdDmxMax = 512;
    gdMiniDmxOn = 0;
    gLedBlink = 0;
	
	SysTick_Config(SystemCoreClock/1000);

    msDelay(100);

//start Independent Watchdog  **************
    IWDG->KR = 0x00005555;
    IWDG->RLR = 0x0000FFFF;
    msDelay(2);
    IWDG->PR = 0x00000002;      //resulting timeout ca. 1-2 seconds
    msDelay(2);
    IWDG->KR = 0x0000CCCC;      //start Independent Watchdog

	msDelay(500);
	gTest = 0;
	if ((GPIOA->IDR & 0x00000010) == 0)  {			//PA4 shorted to Ground
//then reset global parameters to default
		gTest = 1;
		pushFlashPage13(0xA5E7B3C9);	//erase only
		msDelay(500);
	}

//check jumper or front panel switch and decide if ASCII or MIDI commands ****
    GPIOA->MODER &= ~(GPIO_MODER_MODER14);	//repeat input mode
    GPIOA->PUPDR |=  GPIO_PUPDR_PUPDR14_0;	//does not conflict with SWDebug
    GPIOA->PUPDR &=  ~GPIO_PUPDR_PUPDR14_1;
    msDelay(10);
//if PortA,14 = high (default): CDC_CLASS
//if PortA,14 = low (Jumper set): MIDI_CLASS
    guDeviceClass = CDC_CLASS;
    if ((GPIOA->IDR & GPIO_IDR_14) == 0) {
        guDeviceClass = MIDI_CLASS;
    }

#ifdef DMX_DEBUG
	initUsart2();
    msDelay(20);
    sendB2('@');
#endif

//init I/O state  *************************
	popFlashPage13(); //here for USB Vid/Pid only, else see  below
 //init communication  ********************
#ifndef DMX_DEBUG
// init DMX transmitter
	popDmxFlash(0);
	GPIOA->BSRR = GPIO_BSRR_BS_3;			//set bit PA3 (enable DMX driver)
	msDelay(10);	
	startDmxTx();							//transmits even in Setup Mode
	DmxHeader();
	msDelay(10);
#else 
	sendX16(gVid);
	sendX16(gPid);
#endif 
	guConfigured = 0;
	initUSB();
	gTIMEOUT = 5000;
	while (gTIMEOUT)  {
		servSystem();   //serve watchdog etc.
		if (guConfigured)  break;
	}
	msDelay(10);

    gcPushByte = 0;

    while(1)
    {
        servSystem();                       //essential, controls DMX timing
        if (guDeviceClass == CDC_CLASS)
        {
            if (gcPushByte)
            {
				recvbyte = gcPushByte;      //uses next command code as
				gcPushByte = 0;             //flexible parameter input delimiter
				ASCII_Interpreter(recvbyte);
			}
			recvword = pullUsbRxBuf();
			if (recvword)
			{
				recvbyte = (uint8_t)(recvword & 0x00FF);
                if (recvbyte == 'Z')
                {
                    gTIMEOUT = 1000;
                    gdFaderBusy = 0;        //block fade process
                    MiniDMX_Interpreter();
                }
                else  ASCII_Interpreter(recvbyte);
            }
            else if ((gTIMEOUT == 0) && gdMiniDmxOn)
			{
                gdMiniDmxOn = 0;            //terminate MiniDMX
                gLedBlink = 0;
            }
        }
        else {
            recvword = pullUsbRxBuf();
            if (recvword)  {
                status = (uint8_t)(recvword & 0x00FF);
#ifdef DMX_DEBUG
				sendB2('s');
				sendX8(status);
#endif
                index = (status >> 4) & 0x0000000F;
                if ((index > 0x0E) || (index == 0x0A)|| (index == 0x0D)) 
																	index = 0;

                recvword = pullUsbRxBuf();
                if (recvword)  {
					data1 = (uint8_t)(recvword & 0x00FF);
#ifdef DMX_DEBUG
					sendX8(data1);
#endif
                }
                else index = 0;
				if ((index != 0x0C) && (index != CRR)) {
					recvword = pullUsbRxBuf();
					if (recvword)  {
						data2 = (uint8_t)(recvword & 0x00FF);
#ifdef DMX_DEBUG
						sendX8(data2);
#endif
					}
					else  index = 0;
				}
				else data2 = 0;	//formally
#ifdef DMX_DEBUG
						sendB2('i');
						sendX8(index);
#endif

                if (index) MIDI_Interpreter(index,status,data1,data2);
            }
        }
    }
}

//##########################################################################

// Misc Low Level Routines
//************************

void LED_Off(void)  // ******************************************************
// built as subroutine to supply service to "usb-fs-431.c" hardware independent
// **************************************************************************
{
        GPIOA->BSRR = GPIO_BSRR_BR_4; // clear bit PA4,LED OFF
        gLEDCOUNT = 200; //milliseconds
}


void msDelay(uint32_t count)   // *******************************************
//blocking delay by 'count' milliseconds
// **************************************************************************
{
    gDELAYCOUNT = count;
    while(gDELAYCOUNT)  {
        IWDG->KR = 0x0000AAAA;  //timeout ca. 1-2 seconds
    }
}


void wait5us(void)   // *****************************************************
// blocking delay ca 5us
// *************************************************************************
{
uint32_t countdown;

	countdown = 51;
    while (countdown)   {
        IWDG->KR = 0x0000AAAA;  //timeout ca. 1-2 seconds
        countdown-- ;
    }
}


void servSystem(void)   // ************************************************
//system relevant, but not extremely time critical services
//are handled quasi in the background
//exclusively called in pullUsartRxBuf and pullUsbRxBuf
//this way it is regularily served when idle waiting for external input
//but watchdog resets when it hangs somewhere else
//additionally slowed down by SysTick to 50 times/second
// ****************************************************************************
{
uint8_t classChange;
static uint32_t secCount;

//max. high speed part:
#ifndef DMX_DEBUG
	if (USART2->ISR & USART_ISR_TXE) {
		USART2->TDR = gdDMXBUF[gdDmxTxIndex];
// the TXE bit is cleared automatically when TDR is written
		gdDmxTxIndex++;
		if (gdDmxTxIndex >= gdDmxMax)  DmxHeader() ;
	}
#endif
//lower speed services:  **********************************

	if (gSYSCOUNT == 0)  {		//countdown in SysTick_Handler()
		gSYSCOUNT = 20;			//restart 50Hz service interval

		IWDG->KR = 0x0000AAAA;  //fed watchdog - timeout ca. 1-2 seconds

//handle DMX faders
		if (gdFaderBusy)  fadeProcess();

//LED control
		if (gLedBlink)  {
			if (gLedBlink < 6 ) GPIOA->BSRR = GPIO_BSRR_BS_4;
			else GPIOA->BSRR = GPIO_BSRR_BR_4;
			gLedBlink++ ;
			if (gLedBlink >= 12) gLedBlink = 1;
		}
		else if (gLEDCOUNT==0)  GPIOA->BSRR = GPIO_BSRR_BS_4;
													// Set bit PA4 (LED ON)

//very low speed service (1 per sec)    *******************
        secCount ++;
        if (secCount >= 50)  {
            secCount = 0;

			if (gTest) sendUsbByte('R');		//message reset of Globals
/*
#ifndef DMX_DEBUG
//check panel switch or jumper and force new USB enumeration if necessary
//switch open: = (default) ASCII commands, Mini DMX via USB
//switch closed: =  MIDI commands via USB
            classChange = 0;
            if (GPIOA->IDR & GPIO_IDR_14)  {        //then switch is open
                if (guDeviceClass == MIDI_CLASS) {
                    stopUSB();
                    guDeviceClass = CDC_CLASS;
                    classChange = 0xFF;
                }
            }
            else if (guDeviceClass == CDC_CLASS) {
                stopUSB();
                guDeviceClass = MIDI_CLASS;
                classChange = 0xFF;
            }
            if (classChange)  {
                initUSB();
                gTIMEOUT = 5000;                    //5 seconds
                while (gTIMEOUT)  {
                    IWDG->KR = 0x0000AAAA;          //feed watchdog
                    if (guConfigured)  break;
                }
            }
#endif
*/
        }
    }
}


void popFlashPage13(void)   //***********************************************
// **************************************************************************
{
uint32_t temp;
uint32_t  *flashPtr;

	flashPtr = (uint32_t *)PAGE13;

	temp = ~(*flashPtr);			//cannot be==0 if saved project in Flash
	flashPtr++;

	gVid = (temp >> 16) & 0x0000FFFF ;			//cannot be==0 if saved project in Flash


	temp =  ~(*flashPtr);			//cannot be==0 if saved project in Flash
	flashPtr++;
	gPid = temp & 0x0000FFFF ;
	gChannel = (temp >> 16) & 0x0000FFFF;

	if ((temp == 0) || (gVid == 0) || (gPid== 0))  {
		gVid = DEFAULT_VID;			//virgin flash: load default parameters
		gPid = DEFAULT_PID;
		gChannel = 0;
		gdDmxMax  =512;
		return;
	}

	temp = ~(*flashPtr);
	flashPtr++;
	gdDmxMax =temp & 0x0000FFFF;

	if (gChannel > 15) gChannel = 15;
	if (gdDmxMax == 0)  gdDmxMax = 512;		//virgin flash
	if (gdDmxMax < 24)  gdDmxMax = 24;
	if (gdDmxMax > 512)  gdDmxMax = 512;
}

//uint32_t pushFlashPage13(uint32_t mode)   //*********************************
void pushFlashPage13(uint32_t mode)   //*************************************
//SIMPLIFIED (less safe) due to lack of Flash
// **************************************************************************
{
uint32_t k;
uint16_t  *flashPtr;


//unlock flash
#define KEY1    0x45670123      //mandatory unlock key defined by STM
#define KEY2    0xCDEF89AB
    while ((FLASH->SR & FLASH_SR_BSY) !=0);//wait until no operation is going on
    if ((FLASH->CR & FLASH_CR_LOCK) != 0) {
            FLASH->KEYR = KEY1;
            FLASH->KEYR = KEY2;
    }

//erase Page13
    FLASH->CR |= FLASH_CR_PER;
    FLASH->AR = PAGE13;
    FLASH->CR |= FLASH_CR_STRT;
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
    if ((FLASH->SR & FLASH_SR_EOP) !=0)  FLASH->SR |= FLASH_SR_EOP;
//    else return 0;
    FLASH->CR &= ~FLASH_CR_PER;

	msDelay(20);
	if (mode == 0xA5E7B3C9)  {  //erase only requested
//lock flash
		FLASH->CR |= FLASH_CR_LOCK;
		msDelay(100);
//		return  1;		
		return;
	}

    FLASH->CR |= FLASH_CR_PG;
//write system parameters and gfUserToken[] into page29
//programming Flash is possible only by 16bit half-words
//first programmed (even address) 16bit half-word will be read as lower 16bits
//                                          when data are read as 32bit words!!
//second programmed 16bit half-word will be read as higher 16bits
//                                          when data are read as 32bit words!!

	flashPtr = (uint16_t *)PAGE13;

	*flashPtr = 0;		//for virgin test
	flashPtr++;
	while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;

	*flashPtr = (uint16_t)(~gVid & 0x0000FFFF);
	flashPtr++;
	while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
//	else return 0;

	*flashPtr = (uint16_t)(~gPid & 0x0000FFFF);
	flashPtr++;
	while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
//	else return 0;

	*flashPtr = (uint16_t)(~gChannel & 0x0000FFFF);
	flashPtr++;
	while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
//	else return 0;

	*flashPtr = (uint16_t)(~gdDmxMax & 0x0000FFFF);
	flashPtr++;
	while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
//	else return 0;

	*flashPtr =  0xFFFF;        //dummy for 32bit Flash read adjustment
	flashPtr++;
	while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
//	else return 0;

//terminate
	FLASH->CR &= ~FLASH_CR_PG;

//lock flash
	FLASH->CR |= FLASH_CR_LOCK;
	msDelay(10);
//	return 1;
	return;
}


void popDmxFlash(uint32_t preset)  // ***************************************
// preset==0: all channels black. (1 <= preset <= 80) load from Flash
// *************************************************************************
{
uint32_t k, n,page, offset;
uint32_t * flashPtr;
uint32_t temp;

	if (preset == 0)  {
		for (k=0; k < DMX_BUFLEN;k++)  gdFINALMEM[k] = 0;//"dark" scene
	}
	else  {
		preset--;
		page = preset >> 1;					//1k block 0,1,2,...12
		offset = (preset- (page<<1)) * 512;	//0,512 ->2 DMX presets in 1 page
		flashPtr = (uint32_t *)(PAGE14 + (1024 * page) + offset);
		for (k=0; k < 512;k+=4)  {
			n = k>>2;
			temp = ~(*flashPtr);
			gdFINALMEM[k] = temp & 0x000000FF;
			gdFINALMEM[k+1] = (temp >> 8) & 0x000000FF;
			gdFINALMEM[k+2] = (temp >> 16 & 0x000000FF);
			gdFINALMEM[k+3] = (temp >> 24 & 0x000000FF);
			flashPtr++;
		}
	}
	for (k=0;k < DMX_BUFLEN;k++)  {
		if (gdDMXBUF[k] == gdFINALMEM[k]) continue;
		if (gdFadeTime == 0)  gdDMXBUF[k] = gdFINALMEM[k];
		else startFader(k);
	}
}


uint32_t pushDmxFlash(uint32_t preset)   //**********************************
// **************************************************************************
{
uint32_t k, n, page, offset;
uint16_t * flashPtr;

	if (preset == 0)  return 1;				//preset#0 is all channels dark=0
	preset--;

	page = preset >> 1;						//1k block 0,1,2,...12
	offset = (preset- (page<<1)) * 256;		//0,256 ->2 DMX presets in 1 page
	flashPtr = (uint16_t *)(PAGE14 + (page*1024));

//512 bytes must be located into 1k page and 1k page restored back to Flash
	for (k=0; k<512;k++)  {
		gTempBuf[k] = ~(*flashPtr);
		flashPtr++;
	}

	for (n=0;n < 512;n+=2)  {
		k = n>>1;
		gTempBuf[offset+k] = (gdDMXBUF[n] | (gdDMXBUF[n+1] << 8));
	}

//unlock flash
#define KEY1    0x45670123      //mandatory unlock key defined by STM
#define KEY2    0xCDEF89AB

    while ((FLASH->SR &FLASH_SR_BSY) !=0); //wait until no operation is going on
    if ((FLASH->CR & FLASH_CR_LOCK) != 0) {
		FLASH->KEYR = KEY1;
		FLASH->KEYR = KEY2;
	}

//feed watchdog
	IWDG->KR = 0x0000AAAA;          //timeout ca. 1-2 seconds

//erase (Page14 + page)
	FLASH->CR |= FLASH_CR_PER;
	FLASH->AR = PAGE14 + (page * 1024);
	FLASH->CR |= FLASH_CR_STRT;
	while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) !=0)  FLASH->SR |= FLASH_SR_EOP;
	else {
		FLASH->CR &= ~ (FLASH_CR_PER | FLASH_CR_LOCK);
		return 0;
	}
	FLASH->CR &= ~FLASH_CR_PER;

//write system parameters and gfUserToken[] into page29+ff.
//programming Flash is possible only by 16bit half-words
//first programmed (even address) 16bit half-word will be read as lower 16bits
//                                          when data are read as 32bit words!!
//second programmed 16bit half-word will be read as higher 16bits
//                                          when data are read as 32bit words!!
	FLASH->CR |= FLASH_CR_PG;
	flashPtr = (uint16_t *)(PAGE14 + (page * 1024));
	for (k = 0; k < 512; k ++)  {
		*flashPtr = ~(gTempBuf[k]);
		flashPtr++;
//burn
		while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
		if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
		else {
			FLASH->CR &= ~ (FLASH_CR_PG | FLASH_CR_LOCK);
			return 0;
		}
	IWDG->KR = 0x0000AAAA;          //timeout ca. 1-2 seconds
	}
//terminate
	FLASH->CR &= ~FLASH_CR_PG;

//lock flash
	FLASH->CR |= FLASH_CR_LOCK;
	msDelay(10);
	return 1;
}

//###########################################################################

#ifdef DMX_DEBUG
//USART Support
//*************
void initUsart2(void) // ****************************************************
// USART2-TxD is PortA,2
// USART2-RxD is PortA,3
// **************************************************************************
{
    RCC->APB1ENR |=  RCC_APB1ENR_USART2EN ;   // USART2 clock enable

	GPIOA->MODER &= ~(GPIO_MODER_MODER2);
	GPIOA->MODER |= (GPIO_MODER_MODER2_1); //Set pin2 in alternate function mode
                                            // 10=Alternate function mode
    //GPIOA->OTYPER is push-pull by default
    //GPIOA->OSPEEDR is 2 MHz by default
    //GPIOA->PUPDR is HiZ by default (except PA13,14,15)

    GPIOA->BSRR = GPIO_BSRR_BS_3;           //set bit PA3 (enable DMX driver)
    GPIOA->MODER &= ~GPIO_MODER_MODER1;     //PortA1 input in this special case

//USART2 is AF1
    GPIOA->AFR[0] &= 0xFFFF00FF;
    GPIOA->AFR[0] |= 0x00001100;

    USART2->BRR =0x01A1;        //115200Baud: 416.66 at 48 MHz APB1 Clock
    USART2->CR2 = 0;			//default:1 Stopbit
    USART2->CR3 = 0;			//default:no Handshake
    USART2->CR1 = USART_CR1_UE | USART_CR1_TE;      //only TX, 8Bit, no Parity
}


void sendB2(uint8_t data) // *************************************************
//send 1 raw byte
// **************************************************************************
{
    while(!(USART2->ISR&USART_ISR_TXE));
    USART2->TDR = data;
}


void sendX8(uint8_t data)  // **************___******************************
// **************************************************************************
{
uint8_t temp;

    temp= (data&0xF0) >> 4;
    temp = temp+0x30;
    if (temp>0x39) temp+= 7;
    sendB2(temp);
    temp= data&0x0F;
    temp = temp+0x30;
    if (temp>0x39) temp+= 7;
    sendB2(temp);
    sendB2(0x20);
}


void sendNibble(uint8_t data) // ********************************************
// **************************************************************************

{
uint8_t asciiNibble;

    asciiNibble = data+0x30;
    if (asciiNibble>0x39) asciiNibble+= 7;
    sendB2(asciiNibble);
}


void sendX16(uint16_t data) // *******************************************
// **************************************************************************
{
uint8_t nibble;

    nibble = (uint8_t) ((data>>12) & 0x000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>8) & 0x000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>4) & 0x000F);
    sendNibble(nibble);
    nibble = (uint8_t) (data & 0x000F);
    sendNibble(nibble);
    sendB2(0x20);
}


void sendX32(uint32_t data) // **********************************************
// **************************************************************************
{
uint8_t nibble;

    nibble = (uint8_t) ((data>>28) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>24) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>20) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>16) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>12) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>8) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>4) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) (data & 0x0000000F);
    sendNibble(nibble);
    sendB2(0x20);
}
#endif


void sendUsbString(const uint8_t * str) // *************************************
// **************************************************************************
{
 uint8_t byte;

    while  ((byte = *str++) != 0)  sendUsbByte(byte);
}


void sendUsbX8(uint8_t data) // *********************************************
// send 1 byte in hex format (without leading 0x)
// **************************************************************************
{
    uint8_t temp;

    temp= (data&0xF0) >> 4;
    temp = temp+0x30;
    if (temp>0x39) temp+= 7;
    sendUsbByte(temp);
    temp= data&0x0F;
    temp = temp+0x30;
    if (temp>0x39) temp+= 7;
    sendUsbByte(temp);
    sendUsbByte(0x20);
}


void sendUsbNibble(uint8_t data) // ********************************************
// **************************************************************************

{
uint8_t asciiNibble;

    asciiNibble = data+0x30;
    if (asciiNibble>0x39) asciiNibble+= 7;
    sendUsbByte(asciiNibble);
}


void sendUsbX16(uint16_t data) // *******************************************
// **************************************************************************
{
uint8_t nibble;

    nibble = (uint8_t) ((data>>12) & 0x000F);
    sendUsbNibble(nibble);
    nibble = (uint8_t) ((data>>8) & 0x000F);
    sendUsbNibble(nibble);
    nibble = (uint8_t) ((data>>4) & 0x000F);
    sendUsbNibble(nibble);
    nibble = (uint8_t) (data & 0x000F);
    sendUsbNibble(nibble);
    sendUsbByte(0x20);
}


void sendUsbX32(uint32_t data) // **********************************************
// send 1 word (32bit) in hex format (without leading 0x)
// **************************************************************************
{
uint8_t nibble;
    nibble = (uint8_t) ((data>>28) & 0x0000000F);
    sendUsbNibble(nibble);
    nibble = (uint8_t) ((data>>24) & 0x0000000F);
    sendUsbNibble(nibble);
    nibble = (uint8_t) ((data>>20) & 0x0000000F);
    sendUsbNibble(nibble);
    nibble = (uint8_t) ((data>>16) & 0x0000000F);
    sendUsbNibble(nibble);
    nibble = (uint8_t) ((data>>12) & 0x0000000F);
    sendUsbNibble(nibble);
    nibble = (uint8_t) ((data>>8) & 0x0000000F);
    sendUsbNibble(nibble);
    nibble = (uint8_t) ((data>>4) & 0x0000000F);
    sendUsbNibble(nibble);
    nibble = (uint8_t) (data & 0x0000000F);
    sendUsbNibble(nibble);
    sendUsbByte(0x20);

}


void sendUsbS32(uint32_t data) // *******************************************
//send signed decimal number string. Leading zeroes are suppressed
//trailing SPACE
//up to  +/- 2.147.483.647 Billions
// **************************************************************************
{
    sendUsbN32(data);
    sendUsbByte(0x20);
}


void sendUsbN32(uint32_t data) // *******************************************
//send signed decimal number string. Leading zeroes are suppressed
//no leading and trailing SPACE
//up to  +/- 2.147.483.647 Billions
// **************************************************************************
{
uint32_t quotient;

    if (data >= 0x80000000) {
        sendUsbByte('-');
        data = ~data + 1 ;
    }
    if (data < 10) goto ONES;
    if (data < 100) goto TENS;
    if (data < 1000) goto HUNDREDS;
    if (data < 10000) goto THOUSANDS;
    if (data < 100000) goto TH10;
    if (data < 1000000) goto TH100;
    if (data < 10000000) goto MILLIONS;
    if (data < 100000000) goto MIL10;
    if (data < 1000000000) goto MIL100;
    quotient = data/1000000000;
    data = data - quotient*1000000000;
    sendUsbByte((uint8_t)quotient+ 0x30);
MIL100:
    quotient = data/100000000;
    data = data - quotient*100000000;
    sendUsbByte((uint8_t)quotient + 0x30);
MIL10:
    quotient = data/10000000;
    data = data - quotient*10000000;
    sendUsbByte((uint8_t)quotient + 0x30);
MILLIONS:
    quotient = data/1000000;
    data = data - quotient*1000000;
    sendUsbByte((uint8_t)quotient + 0x30);
TH100:
    quotient = data/100000;
    data = data - quotient*100000;
    sendUsbByte((uint8_t)quotient+ 0x30);
TH10:
    quotient = data/10000;
    data = data - quotient*10000;
    sendUsbByte((uint8_t)quotient + 0x30);
THOUSANDS:
    quotient = data/1000;
    data = data - quotient*1000;
    sendUsbByte((uint8_t)quotient + 0x30);
HUNDREDS:
    quotient = data/100;
    data = data - quotient*100;
    sendUsbByte((uint8_t)quotient+ 0x30);
TENS:
    quotient = data/10;
    data = data - quotient*10;
    sendUsbByte((uint8_t)quotient + 0x30);
ONES:
    sendUsbByte((uint8_t)data + 0x30) ;
}


// Interrupt Handler
//******************

void SysTick_Handler(void)  // ***********************************************
// ***************************************************************************
{
	if ((gSYSCOUNT > 0)&&(gSYSCOUNT < 0x8000)) gSYSCOUNT--; //controls servSystem()
	else gSYSCOUNT = 0;             //double safety against underflow
	if (gLEDCOUNT > 0) gLEDCOUNT--;
	if (gDELAYCOUNT > 0) gDELAYCOUNT--;
	if (gTIMEOUT > 0) gTIMEOUT--;   //to be used in any subroutine
}

// #########################################################################

