// main.h Header file for **********************************
// simple DMX512 controller for STM32F042F6 (20 pin TSSOP package)
// based on USB MIDISTREAM and CDC-ABSTRACT CONTROL MODEL
//
// Author and (c)2017-21 Wolfgang Schemmert.
// The copyright claim and GPL-3 licence is restricted to this file
// and the depending files "main.c","usb-fs-042.c","dmx.c","userinterface.c"
// and "globaldefines.h".
// Other depending material is published and copyrighted
// by STMicroelectronics and Segger GmbH/Rowley Associates Ltd
//
// Contact: <www.midi-and-more.de>
// A detailled hardware construction and operation manual
// is provided at this website
//
// ****************************************************************************
//      This program is free software: you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//      Licence: GPL-3
//
// ****************************************************************************
#include "globaldefines.h"

// extern defined in userinterface.c  ************
extern uint8_t gcPushByte, gcSetupMode;
extern void  ASCII_Interpreter(uint8_t);
extern void MIDI_Interpreter(uint8_t,uint8_t,uint8_t,uint8_t);
extern uint8_t Setup_Interpreter(uint8_t);
extern void copyComParams(void);

// from usb-fs.c
extern uint32_t guDeviceClass, guConfigured;
extern void stopUSB(void);
extern void initUSB(void);
extern uint32_t pullUsbRxBuf(void);
extern void sendUsbByte(uint8_t);
extern uint32_t pullMidiMsg(void);

// from dmx.c
extern uint8_t gdDMXBUF[DMX_BUFLEN+2];
extern uint8_t gdFINALMEM[DMX_BUFLEN+2];
extern uint32_t gdDmxAddr, gdDmxTxIndex, gdDmxMax;
extern uint32_t gdFaderBusy, gdMiniDmxOn;
extern uint32_t gdFadeTime;
extern void startDmxTx(void);
extern void DmxHeader(void);
extern void MiniDMX_Interpreter(void);
extern void fadeProcess(void);
extern void startFader(uint16_t);

// ****************************************

//system control
uint32_t gSYSCOUNT, gDELAYCOUNT, gTIMEOUT, gLEDCOUNT, gLedBlink;
uint32_t gVid, gPid, gChannel, gTest;

#define PAGE13 (0x08003400)		//page13 reserved for global parameters
#define PAGE14 (0x08003800)
//pages 14..31 reserved for DMX, 18kB totally = 36 presets
uint16_t gTempBuf[512];

void LED_Off(void);
void msDelay(uint32_t);
void wait5us(void);
void servSystem(void);
void popFlashPage13(void);
//uint32_t pushFlashPage13(uint32_t);
void pushFlashPage13(uint32_t);
void popDmxFlash(uint32_t);
uint32_t pushDmxFlash(uint32_t);


#ifdef DMX_DEBUG
void initUsart2(); 
void sendB2(uint8_t);
void sendNibble(uint8_t);
void sendX8(uint8_t);
void sendX16(uint16_t);
void sendX32(uint32_t);
#endif

void sendUsbString(const uint8_t *);
void sendUsbNibble(uint8_t);
void sendUsbX8(uint8_t);
void sendUsbX16(uint16_t);
void sendUsbX32(uint32_t);
void sendLS32(uint32_t);
void sendUsbS32(uint32_t);
void sendUsbN32(uint32_t);
uint32_t pullUsartRxBuf(void);
uint32_t pullTermRxBuf(void);
uint32_t pullIORxBuf(void);
uint8_t parseHexNibble(uint8_t);
uint32_t parseHexNum(uint8_t);
uint8_t parseDeciDigit(void);
//Interrupt handlers
//void SysTick_Handler(void);
//void USART2_IRQHandler(void);
