// userinterface.c.c  // ***************************************************************
//
// user control related files for
// simple DMX512 controller for STM32F042F6 (20 pin TSSOP package)
// based on USB MIDISTREAM and CDC-ABSTRACT CONTROL MODEL
//
// Author and (c)2017-21 Wolfgang Schemmert.
// The copyright claim and GPL-3 licence is restricted to this file
// and the depending files "main.c","main.h","usb-fs-042.c", "dmx.c"
// and "globaldefines.h".
// Other depending material is published and copyrighted
// by STMicroelectronics and Segger GmbH/Rowley Associates Ltd
//
// Contact: <www.midi-and-more.de>
// A detailled hardware construction
// and operation manual is provided at this website
//
// ****************************************************************************
//      This program is free software: you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//      Licence: GPL-3
//
// ****************************************************************************

#include "stm32f0xx.h"
#include "globaldefines.h"

// ************************************

//main.h
#define PAGE14 (0x08003800)
extern uint16_t gTempBuf[512];
extern uint32_t gVid, gPid, gChannel, gLedBlink, gTest;
extern void msDelay(uint32_t);
extern void popFlashPage13(void);
//extern uint32_t pushFlashPage13(uint32_t);
extern void pushFlashPage13(uint32_t);
extern void popDmxFlash(uint32_t);
extern uint32_t pushDmxFlash(uint32_t);

#ifdef DMX_DEBUG
extern void sendB2(uint8_t);
extern void sendX8(uint8_t);
extern void sendX16(uint16_t);
extern void sendX32(uint32_t);
#endif

extern void sendUsbX8(uint8_t);
extern void sendUsbN8(uint8_t);
extern void sendUsbX16(uint16_t);
extern void sendUsbX32(uint32_t);
extern void sendUsbN32(uint32_t);
extern void sendUsbS32(uint32_t);
extern void sendUsbString(const uint8_t *);

//usb-fs.c
extern uint8_t guDeviceClass;
extern uint16_t pullUsbRxBuf(void);
extern void sendUsbByte(uint8_t);

//dmx.c
extern uint8_t gdDMXBUF[DMX_BUFLEN+2];          //actual DMX transmit buffer
extern uint8_t gdFINALMEM[DMX_BUFLEN+2];        //final level of fade process
extern uint32_t gdDmxAddr, gdFadeTime, gdDmxMax;
extern void setDmxAddress(void);
extern void setDmxLevel(uint8_t);
extern void setNextDmxLevel(uint8_t);
extern void setEqual(uint8_t);
extern void msgDmxState(uint32_t);
extern void readDmxBuf(uint32_t);
extern void sendAddrLevel(void);
extern void sendLevel(uint16_t);

// ************************************


uint8_t gcPushByte;
uint32_t gcIgnoreVelo0, gcSetupMode;

const uint8_t gcSetStr[] = {"  set"};
const uint8_t gcVidPidStr[] = {"USB Vid/Pid of this device (hex):"};
const uint8_t gcChannelStr[] = {"MIDI base channel (1-16):"};
const uint8_t gcConfirmStr[] = {"Save Setup to Flash?->press upper case 'Y'"};
const uint8_t gcErrorStr[] = {"Flash Write Error"};
const uint8_t gcFadeStr[] = {"FadeTime:"};
const uint8_t gcCycleStr[] = {"DmxCycle:"};

void  ASCII_Interpreter(uint8_t);
void MIDI_Interpreter(uint8_t,uint8_t,uint8_t,uint8_t);
void  Setup_Interpreter(uint8_t);
uint32_t parseHexByte(void);
uint32_t parseHexNibble(void);
uint32_t parseDeciNum(uint32_t);
uint8_t parseDeciNibble(void);
//void sendS3(uint32_t);
void sendS2(uint32_t);
uint32_t setVidPid(void);
void setMidiChannel(void);
void listSetup(void);


void ASCII_Interpreter(uint8_t CmdCode)  // ******************************
// **************************************************************************
{
uint32_t k;
uint32_t recvword;
const uint8_t againStr[] = {"all lights DARK?->press again "};

    gLedBlink = 0;

    if ((CmdCode > 0x60) && (CmdCode < 0x7B)) CmdCode -= 0x20; //->upper case

    if (CmdCode > 0x20) sendUsbByte(CmdCode);

    switch (CmdCode)  {
    case 'S':
        setDmxAddress();
        break;

    case 'V':
        if ((k = parseDeciNum(255)) > 255)  {
            sendUsbByte('?');
            gcPushByte = 0;
        }
        else setDmxLevel(k);
        break;

    case 0x2C:      //comma
         if ((k = parseDeciNum(255)) > 255)  {
            sendUsbByte('?');
            gcPushByte = 0;
        }
        else setNextDmxLevel(k);
        break;

    case 'Q':
        msgDmxState(1);
        break;

    case 'R':
        readDmxBuf(0);
        break;

    case 'T':
        if ((k = parseDeciNum(127)) > 127)  {
            sendUsbByte('?');
            gcPushByte = 0;
        }
        else gdFadeTime = k * 5;
        break;

    case '+':
        if (gdDMXBUF[gdDmxAddr] != 0xFF) {
            gdDMXBUF[gdDmxAddr]++ ;
            gdFINALMEM[gdDmxAddr]++ ;
        }
        break;

	case '-':
        if (gdDMXBUF[gdDmxAddr] != 0x00) {
            gdDMXBUF[gdDmxAddr]-- ;
            gdFINALMEM[gdDmxAddr]-- ;
        }
        break;

    case '=':
        if ((k = parseDeciNum(511)) > 511)  {
            sendUsbByte('?');
            gcPushByte = 0;
        }
        else setEqual(k);
        break;

	case 'X':
		for (k=0; k < gdDmxMax;k++)  {
			gdFINALMEM[k] = gdDMXBUF[k]; //store actual values into end-of-Fade mem
		}
		break;
   case 'L':

		k = parseDeciNum(512);
        if ((k > 512) || (k < 24))  {
            sendUsbByte('?');
            gcPushByte = 0;
        }
        else  {
			gdDmxMax = k;
			pushFlashPage13(1);
        }
        break;

    case 'D':
		sendUsbByte(CRR);
		sendUsbString(againStr);
		msDelay(500);						//ensure intended handling
		do  {
			recvword = pullUsbRxBuf();
		}  while (recvword == 0) ;
		if ((recvword == 0x8044) || (recvword == 0x8064)) { //'d' or 'D'
			for (k=0; k<512; k++)  {
				gdDMXBUF[k] = 0 ;
				gdFINALMEM[k] = 0;
			}
		}
		else  {
			sendUsbByte('?');
		gcPushByte = 0;
		}
		break;

	case 'C':
		sendUsbString(gcSetStr);
		sendUsbString(gcChannelStr);
		setMidiChannel();
		break;

	case 'U':
		sendUsbString(gcSetStr);
		sendUsbString(gcVidPidStr);
		if (setVidPid() == 0) sendUsbByte('?');
		break;

    case '@':
        if ((k = parseDeciNum(36)) > 36)  {
            sendUsbByte('?');
            gcPushByte = 0;
        }
		else  popDmxFlash(k);
        break;

    case '~':
        if ((k = parseDeciNum(36)) > 36)  {
            sendUsbByte('?');
            gcPushByte = 0;
        }
		else {
			if (pushDmxFlash(k) == 0) sendUsbString(gcErrorStr);
		}
		break;

     case '?':                   //list actual configuration
		gTest = 0;
		listSetup();
		break;

    case 0x0A:
		sendUsbByte(CRR);
    	sendUsbByte('>');
        return;
    case 0x0D:
		sendUsbByte(CRR);
    	sendUsbByte('>');
        return;
    case 0x20:
        break;      //ignore SPACE
    default:
        sendUsbByte('?');
        gcPushByte = 0;
        break;
    }
	sendUsbByte(CRR);
	sendUsbByte('>');
}


void MIDI_Interpreter(uint8_t index,uint8_t status,uint8_t data1,uint8_t data2)
// ****************************************************************************
{
uint8_t channel;
uint32_t k;

#ifdef DMX_DEBUG
    sendX8(index);
    sendX8(status);
    sendX8(data1);
    sendX8(data2);
#endif

    gLedBlink = 0;

    if (index == 0x08)  {
        index = 0x09;
        data2 = 0;                          //with zero velocity
    }
    channel = status & 0x0F;
    if (index == 0x09){
        if ((channel < gChannel) || (channel >  (gChannel + 3)))  return;
    }
    else if (channel != gChannel) return;

    switch (index)  {
    case 0x09:
        if ((data2 == 0) && (gcIgnoreVelo0))  break;
        gdDmxAddr = data1 + 128 * (channel - gChannel);
        if (gdDmxAddr == 0) gdDmxAddr = 512;
        gdDmxAddr-- ;				//user DMX slot 1-512 = internal 0-511 !!
        data2 = data2 << 1;                 //adjust level 0 ... 0xFE(254)
        if (data2 == 0xFE) data2 = 0xFF;
        if ((data2 == 2) && (gcIgnoreVelo0)) data2 = 0;
        setDmxLevel(data2);
        break;

    case 0x0B:
        if (data1 == 1) gdFadeTime = data2 * 5; //setFadeTime(data2);
        else if ((data1 >= 0x50) && (data1 < 0x54))  {
            if ((data1 == 0x50) && (data2 == 0x00)) gdDmxAddr = 512;
            else gdDmxAddr = 128 * (data1 - 0x50) + data2;
            gdDmxAddr-- ;			//user DMX slot 1-512 = internal 0-511 !!
        }
        else if (data1 == 0x54)  {
            data2 = data2 <<1;
			setDmxLevel(data2);
        }
        else if (data1 == 0x55)  {
			data2 = (data2 <<1) + 1;
			setDmxLevel(data2);
        }
        else if ((data1 == 0x56) && (gdDmxAddr < 511))  {
            gdDmxAddr++ ;
            data2 = data2 <<1;
            setDmxLevel(data2);
        }
        else if ((data1 == 0x57) && (gdDmxAddr < 511)) {
            gdDmxAddr++ ;
            data2 = (data2 <<1) + 1;
            setDmxLevel(data2);
        }
        else if ((data1 == 0x24) && (data2 != 0)) {
			sendAddrLevel();
			if (data2 == 1)  break;
			data2--;
			if (data2 > 9) data2 = 9;
            for (k=gdDmxAddr+1;k <= (gdDmxAddr+data2);k++) {
                if (k > 512)  break;
				sendLevel(k);
            }
        }
		else if ((data1 == 0x25)&&(data2 != 0)) {
			sendUsbByte(0xF0);
			sendUsbByte(0x7D);
			readDmxBuf((uint32_t)data2);
			sendUsbByte(EOX);
		}
        else if (data1 == 0x28)  setEqual(data2); //Changed for compatibility
        else if (data1 == 0x30)  {		//Changed for compatibility
            if (data2 < 5) break;		//ignore bad input	bugfix 27July20
            else gdDmxMax = (data2+1) << 2;		 //in steps of 4 DMX Tx slots
        }

		else if ((data1 == 0x60) && (data2 <= 36)) popDmxFlash(data2);
		else if ((data1 == 0x70) && (data2 <= 36)) pushDmxFlash(data2);
        else if (data1 == 0x77)  { //changed with respect to earlier versions!
            data1 = data2;
            goto PROGRAM_CHANGE;
        }
        break;

    case 0x0C:
PROGRAM_CHANGE:
		if (data1 == 1)  {	
//stop all Fade processes
			for (k=0; k < gdDmxMax;k++)  {
				gdFINALMEM[k] = gdDMXBUF[k]; //store actual values into end-of-Fade mem
			}
		}
        else if ((data1 == 8) && (gdDMXBUF[gdDmxAddr] != 0x00)) {
            gdDMXBUF[gdDmxAddr]-- ;
            gdFINALMEM[gdDmxAddr]-- ;
        }
        else if ((data1 == 9) && (gdDMXBUF[gdDmxAddr] != 0xFF)) {
            gdDMXBUF[gdDmxAddr]++ ;
            gdFINALMEM[gdDmxAddr]++ ;
        }
		else if (data1 == 0x77)  {
			sendUsbByte(0xF0);
			sendUsbByte(0x7D);
			msgDmxState(0);
			sendUsbByte(EOX);
		}
	    else if (data1 == 0x78) gcIgnoreVelo0 = 0;
        else if (data1 == 0x79) gcIgnoreVelo0 = 0xFF;
        else if (data1 == 0x7F)  {
			for (k=0;k<512;k++)  {
				gdDMXBUF[k] = 0;
				gdFINALMEM[k] = 0;
			}
		}
        break;

	case 0x0E:	//PITCHWHEEL change: data2 = 7MSB, data1,bit6 is LSB
			data2 = data2 << 1;
			if (data1 & 0x40)  data2++;
            setDmxLevel(data2);
		break;

    default:
        break;
    }
}


uint32_t parseHexNibble(void)   // ******************************************
// **************************************************************************
{
uint16_t recvword;
uint32_t nibble;

    while ((recvword = pullUsbRxBuf()) == 0) ;
    nibble = (uint32_t)(recvword & 0x00FF);
    if(nibble == ESC) return 0xFFFFFFFF;	//emergency termination of input
    if ((nibble == 0x0A)||(nibble == 0x0D)||(nibble == '.')) return 0xFFFFFFFE;
    if  ((nibble >= 0x30) && (nibble <= 0x39))  {
        sendUsbByte((uint8_t)nibble);
        return (nibble - 0x30);
    }
    if ((nibble > 0x60) && (nibble < 0x7B))  nibble -= 0x20; //force uppercase
    if ((nibble >= 'A') && (nibble <= 'F')) {
        sendUsbByte((uint8_t)nibble);
        return (nibble - 0x37);
    }
    else return 0xFFFFFFFF;

}


uint32_t  parseHexByte(void)   // ********************************************
// ***************************************************************************
{
uint32_t nibble1, nibble2;

        if ((nibble1 = parseHexNibble())  == 0xFFFFFFFE) return 0xFFFFFFFF;
        else if (nibble1 > 0x0000000F) return 0xFFFFFFFF;
        if ((nibble2 = parseHexNibble()) == 0xFFFFFFFE) return nibble1;
        else if (nibble2 > 0x0000000F)  return 0xFFFFFFFF;
        if (parseHexNibble() != 0xFFFFFFFE) return 0xFFFFFFFF;
        return (uint32_t)((nibble1 << 4) | nibble2);
}


uint32_t parseDeciNum(uint32_t limit)   // **********************************
//limit 0 ...max 99999
// **************************************************************************
{
uint32_t decinum, places ;
uint16_t recvword;
uint8_t digit;
	
    if (limit >= 99000) return 0xFFFFFFFF;
    places = 0;
    decinum = 0;
    while (decinum <= limit)  {
        while ((recvword = pullUsbRxBuf()) == 0);
        digit = (uint8_t)(recvword & 0x00FF);
		if(digit == ESC) return 0xFFFFFFFF;	//emergency termination of input
        if ((digit == 0x0A) || (digit == 0x0D) || (digit == '.')) {
			if (places == 0) return 0xFFFFFFFF;	//probably wrong command
            else return decinum;
        }
        if ((digit < 0x30) || (digit > 0x39 )) {
			gcPushByte = digit;     //flexible delimiter, will be evaluated
                                    //as maybe next command code
            return decinum;
        }
        sendUsbByte(digit);
        digit -= 0x30;     //ASCII>num
        decinum  = (decinum * 10) + digit;
        if (decinum > limit) {
            sendUsbByte('?');
            return 0xFFFFFFFF;
        }
        places++;
    }
    return decinum;
}


uint8_t parseDeciNibble(void)   // ******************************************
// **************************************************************************
{
uint32_t number;

    if ((number = parseDeciNum(15)) > 15) {
        sendUsbByte('?');
        return 0xFF;
    }
    return (uint8_t) number;
}


void sendS2(uint32_t number)   // ************************************************
// **************************************************************************
{
    if (number < 10) sendUsbByte(0x20);
    sendUsbS32(number);
}

uint32_t setVidPid(void)  // ****************************************************
// **************************************************************************
{
uint32_t nibble;
uint32_t recvword, Vid, Pid;

const uint8_t VidStr[] = {"Vid="};
const uint8_t PidStr[] = {"Pid="};

    sendUsbString(VidStr);
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Vid = nibble;
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Vid = (Vid << 4) + nibble;                // *16
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Vid = (Vid << 4) + nibble;
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Vid = (Vid << 4) + nibble;
    sendUsbByte(' ');

    sendUsbString(PidStr);
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Pid = nibble;
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Pid = (Pid << 4) + nibble;
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Pid = (Pid << 4) + nibble;
    if ((nibble = parseHexNibble()) > 0x000000FF) return 0;
    Pid = (Pid << 4) + nibble;

//only write to Shadow after all items are entered correctly
	sendUsbByte(CRR);
	sendUsbString(gcConfirmStr);
	do  {
		recvword = pullUsbRxBuf();
	}  while (recvword == 0) ;
	if (recvword == 0x8059)  {      // 'Y'
		gVid = Vid;
		gPid = Pid;
//		if (pushFlashPage13(1) == 0)  {
//			sendUsbString(gcErrorStr);
//			return 0;
//		}
		pushFlashPage13(1);
		sendUsbByte(0x20);
		sendUsbByte('o');
		sendUsbByte('k');
	}

    return 1;
}


void setMidiChannel(void)  // ***********************************************
//get the common MIDI channel in decimal format 1...16
// **************************************************************************
{
uint32_t recvword, number;
const uint8_t ChannelStr[] = {" 1-16!?"};

    if(((number = parseDeciNum(16)) > 16) || (number == 0)) {
        sendUsbString(ChannelStr);
        return;
    }
	sendUsbByte(CRR);
	sendUsbString(gcConfirmStr);
	do  {
		recvword = pullUsbRxBuf();
	}  while (recvword == 0) ;
	if (recvword == 0x8059)  {      // 'Y'
		gChannel = number - 1;
//		if (pushFlashPage13(1) == 0)  {
//			sendUsbString(gcErrorStr);
//			return;
//		}
		pushFlashPage13(1);
		sendUsbByte(0x20);
		sendUsbByte('o');
		sendUsbByte('k');
	}
}



void listSetup(void)  // *****************************************************
// **************************************************************************
{
uint8_t timeH;
const uint8_t SetupStr[] = {"Global parameters:"};
const uint8_t TypeStr[] = "USB/DMX Controller with STM32F042 v" ;
const uint8_t CopyStr[] = " (c)2017-21 W.Schemmert";

	sendUsbByte(CRR);
	sendUsbString(TypeStr);
	sendUsbByte(HARDWARE_VERSION + 0x30);
	sendUsbByte('.');
	sendUsbByte(REVNUM + 0x30);
	sendUsbString(CopyStr);

	sendUsbByte(CRR);
	sendUsbString(SetupStr);
	sendUsbByte(CRR);
	sendUsbString(gcFadeStr);
	timeH = gdFadeTime/50;
	sendUsbN32(timeH);
	sendUsbByte('.');
	sendUsbN32((gdFadeTime/5) - (timeH*10));
	sendUsbByte('s');
	sendUsbByte(CRR);
	sendUsbString(gcCycleStr);
	sendUsbN32(gdDmxMax);
	sendUsbByte(CRR);
	sendUsbString(gcChannelStr);
	sendS2((gChannel & 0x0000000F) +1);
	sendUsbByte(CRR);
	sendUsbString(gcVidPidStr);
	sendUsbX16(gVid & 0x0000FFFF);
	sendUsbByte('/');
	sendUsbByte(0x20);
	sendUsbX16(gPid & 0x0000FFFF);
}

// ##########################################################################
